package app.navigation.player;

import app.components.label.ValidationLabel;
import app.data.entity.Player;
import app.data.entity.Season;
import app.data.service.PlayerService;
import app.data.service.SeasonService;
import app.navigation.Navigation;
import app.navigation.NavigationUtils;
import app.utils.EntityStringUtils;
import com.vaadin.flow.component.AbstractField.ComponentValueChangeEvent;
import com.vaadin.flow.component.HasValue.ValueChangeListener;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.select.Select;
import com.vaadin.flow.router.BeforeEvent;
import com.vaadin.flow.router.WildcardParameter;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@SuppressWarnings("OptionalUsedAsFieldOrParameterType")
public class PlayerNavigation extends Navigation {


    private final PlayerService playerService;
    private final SeasonService seasonService;

    private final List<Player> playerList = new ArrayList<>();
    private final List<Season> seasonList = new ArrayList<>();

    private final Select<Player> playerSelect = new Select<>();
    private final Select<Season> seasonSelect = new Select<>();


    public PlayerNavigation(@Autowired PlayerService playerService,
                            @Autowired SeasonService seasonService) {
        this.playerService = playerService;
        this.seasonService = seasonService;

        fillPlayerSelectWithData();

        playerSelect.addValueChangeListener(playerSelectValueChangeListener());
        seasonSelect.addValueChangeListener(seasonSelectValueChangeListener());

        playerSelect.setItemLabelGenerator(EntityStringUtils::getPlayerString);
        seasonSelect.setItemLabelGenerator(EntityStringUtils::getSeasonString);
    }

    public void addRunnableToBeRunAfterSelection(Runnable runnable) {
        runnablesToBeRunAfterSelection.add(runnable);
    }

    protected void updateUrl() {
        String playerParam = null;
        String seasonParam = null;
        if (playerSelect.getOptionalValue().isPresent())
            playerParam = EntityStringUtils.getPlayerStringForURL(playerSelect.getValue());
        if (seasonSelect.getOptionalValue().isPresent())
            seasonParam = EntityStringUtils.getSeasonStringForURL(seasonSelect.getValue());

        String params = NavigationUtils.getWildcardParam(false, playerParam, seasonParam);
        UI.getCurrent().getPage().getHistory().pushState(null, String.format("%s/%s", getRoute(), params));
    }

    private ValueChangeListener<ComponentValueChangeEvent<Select<Player>, Player>> playerSelectValueChangeListener() {
        return event -> {
            fillSeasonSelectWithData(event.getValue());
            autoselectSeason();
        };
    }

    private ValueChangeListener<ComponentValueChangeEvent<Select<Season>, Season>> seasonSelectValueChangeListener() {
        return event -> doPostSelectionStuff();
    }

    private void autoselectPlayer() {
        if (playerList.isEmpty()) {
            validationLabel.setText("No Players in List!");
            validationLabel.setValid(false);
            return;
        }
        playerSelect.setValue(playerList.get(0));
    }

    private void autoselectSeason() {
        if (seasonList.isEmpty()) {
            validationLabel.setText("No Season in List!");
            validationLabel.setValid(false);
            return;
        }
        seasonSelect.setValue(seasonList.get(seasonList.size() - 1));
    }

    private void fillPlayerSelectWithData() {
        playerList.clear();
        playerList.addAll(playerService.getAllPlayersSorted());
        playerSelect.setItems(playerList);
    }

    private void fillSeasonSelectWithData(Player player) {
        seasonList.clear();
        seasonList.addAll(seasonService.getAllSeasonsForPlayerSorted(player));
        seasonSelect.setItems(seasonList);
    }

    @Override
    public void setParameter(BeforeEvent event, @WildcardParameter String param) {
        Map<PlayerNavigationLevel, Optional<String>> map = PlayerNavigationUtils.getParameterMap(param);
        navigate(map.get(PlayerNavigationLevel.PLAYER), map.get(PlayerNavigationLevel.SEASON));
    }

    private void navigate(Optional<String> playerParam, Optional<String> seasonParam) {
        Optional<Player> player = NavigationUtils.getObjectFromParam(playerList, playerParam);
        if (player.isPresent()) playerSelect.setValue(player.get());
        else autoselectPlayer();

        Optional<Season> season = NavigationUtils.getObjectFromParam(seasonList, seasonParam);
        if (season.isPresent()) seasonSelect.setValue(season.get());
        else autoselectSeason();
    }

    public List<Player> getPlayerList() {
        return playerList;
    }

    public List<Season> getSeasonList() {
        return seasonList;
    }

    public Select<Player> getPlayerSelect() {
        return playerSelect;
    }

    public Select<Season> getSeasonSelect() {
        return seasonSelect;
    }

    public Optional<Player> getSelectedPlayer() {
        return playerSelect.getOptionalValue();
    }

    public Optional<Season> getSelectedSeason() {
        return seasonSelect.getOptionalValue();
    }

    public PlayerService getPlayerService() {
        return playerService;
    }

    public SeasonService getSeasonService() {
        return seasonService;
    }

    @Override
    public String getWildcardParam() {
        return NavigationUtils.getWildcardParam(
                false,
                getPlayerParam().orElse(null),
                getSeasonParam().orElse(null));
    }

    public Optional<String> getPlayerParam() {
        return getSelectedPlayer().map(EntityStringUtils::getPlayerStringForURL);
    }

    public Optional<String> getSeasonParam() {
        return getSelectedSeason().map(EntityStringUtils::getSeasonStringForURL);
    }
}
