package app.navigation.regular;

import app.data.entity.Match;
import app.data.entity.Matchday;
import app.data.entity.Season;
import app.data.service.MatchService;
import app.data.service.MatchdayService;
import app.data.service.SeasonService;
import app.navigation.Navigation;
import app.navigation.NavigationUtils;
import app.utils.EntityStringUtils;
import com.vaadin.flow.component.AbstractField.ComponentValueChangeEvent;
import com.vaadin.flow.component.HasValue.ValueChangeListener;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.select.Select;
import com.vaadin.flow.router.BeforeEvent;
import com.vaadin.flow.router.WildcardParameter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@SuppressWarnings("OptionalUsedAsFieldOrParameterType")
public class RegularNavigation extends Navigation {

    private boolean onlyMatchdaysWithActivity;

    private final SeasonService seasonService;
    private final MatchdayService matchdayService;
    private final MatchService matchService;

    private final List<Season> seasonList = new ArrayList<>();
    private final List<Matchday> matchdayList = new ArrayList<>();
    private final List<Match> matchList = new ArrayList<>();

    private final Select<Season> seasonSelect = new Select<>();
    private final Select<Matchday> matchdaySelect = new Select<>();
    private final Select<Match> matchSelect = new Select<>();

    private RegularNavigationLevel regularNavigationLevel = RegularNavigationLevel.SEASON;

    public RegularNavigation(@Autowired SeasonService seasonService,
                             @Autowired MatchdayService matchdayService,
                             @Autowired MatchService matchService) {
        this.seasonService = seasonService;
        this.matchdayService = matchdayService;
        this.matchService = matchService;

        fillSeasonSelectWithData();

        seasonSelect.addValueChangeListener(seasonSelectValueChangeListener());
        matchdaySelect.addValueChangeListener(matchdaySelectValueChangeListener());
        matchSelect.addValueChangeListener(matchSelectValueChangeListener());

        seasonSelect.setItemLabelGenerator(EntityStringUtils::getSeasonString);
        matchdaySelect.setItemLabelGenerator(EntityStringUtils::getMatchdayString);
        matchSelect.setItemLabelGenerator(EntityStringUtils::getMatchString);
    }

    public void setOnlyMatchdaysWithActivity(boolean onlyMatchdaysWithActivity) {
        this.onlyMatchdaysWithActivity = onlyMatchdaysWithActivity;
    }

    public void setNavigationLevel(@NonNull RegularNavigationLevel regularNavigationLevel) {
        this.regularNavigationLevel = regularNavigationLevel;
    }

    public void addRunnableToBeRunAfterSelection(Runnable runnable) {
        runnablesToBeRunAfterSelection.add(runnable);
    }

    public boolean seasonEnabled() {
        return this.regularNavigationLevel.compareTo(RegularNavigationLevel.SEASON) >= 0;
    }

    public boolean matchdayEnabled() {
        return this.regularNavigationLevel.compareTo(RegularNavigationLevel.MATCHDAY) >= 0;
    }

    public boolean matchEnabled() {
        return this.regularNavigationLevel.compareTo(RegularNavigationLevel.MATCH) >= 0;
    }

    protected void updateUrl() {
        String seasonParam = null;
        String matchdayParam = null;
        String matchParam = null;
        if (seasonEnabled() && seasonSelect.getOptionalValue().isPresent())
            seasonParam = EntityStringUtils.getSeasonStringForURL(seasonSelect.getValue());
        if (matchdayEnabled() && matchdaySelect.getOptionalValue().isPresent())
            matchdayParam = EntityStringUtils.getMatchdayStringForURL(matchdaySelect.getValue());
        if (matchEnabled() && matchSelect.getOptionalValue().isPresent())
            matchParam = EntityStringUtils.getMatchStringForURL(matchSelect.getValue());

        String params = NavigationUtils.getWildcardParam(editFlag, seasonParam, matchdayParam, matchParam);
        UI.getCurrent().getPage().getHistory().pushState(null, String.format("%s/%s", getRoute(), params));
    }

    private ValueChangeListener<ComponentValueChangeEvent<Select<Season>, Season>> seasonSelectValueChangeListener() {
        return event -> {
            if (!seasonEnabled()) throw new IllegalStateException("Cannot select season when it is not enabled!");
            if (matchdayEnabled()) {
                fillMatchdaySelectWithData(event.getValue());
                autoselectMatchday();
                return;
            }
            doPostSelectionStuff();
        };
    }

    private ValueChangeListener<ComponentValueChangeEvent<Select<Matchday>, Matchday>> matchdaySelectValueChangeListener() {
        return event -> {
            if (!matchdayEnabled()) throw new IllegalStateException("Cannot select matchday when it is not enabled!");
            if (matchEnabled()) {
                fillMatchSelectWithData(event.getValue());
                autoselectMatch();
                return;
            }
            doPostSelectionStuff();
        };
    }

    private ValueChangeListener<ComponentValueChangeEvent<Select<Match>, Match>> matchSelectValueChangeListener() {
        return event -> {
            if (!matchEnabled()) throw new IllegalStateException("Cannot select match when it is not enabled!");
            doPostSelectionStuff();
        };
    }

    private void autoselectSeason() {
        if (!seasonEnabled())
            throw new IllegalStateException("This method should not be called when season is not enabled!");
        if (seasonList.isEmpty()) {
            validationLabel.setText("No Seasons in List!");
            validationLabel.setValid(false);
            return;
        }
        seasonSelect.setValue(seasonList.get(seasonList.size() - 1));
    }

    private void autoselectMatchday() {
        if (!matchdayEnabled())
            throw new IllegalStateException("This method should not be called when matchday is not enabled!");
        if (matchdayList.isEmpty()) {
            validationLabel.setText("No Matchdays in List!");
            validationLabel.setValid(false);
            return;
        }
        Matchday matchdayToSelect = matchdayList.get(0);
        for (Matchday matchday : matchdayList) if (matchdayService.hasActivity(matchday)) matchdayToSelect = matchday;
        matchdaySelect.setValue(matchdayToSelect);
    }

    private void autoselectMatch() {
        if (!matchEnabled())
            throw new IllegalStateException("This method should not be called when match is not enabled!");
        if (matchList.isEmpty()) {
            validationLabel.setText("No Matches in List!");
            validationLabel.setValid(false);
            return;
        }
        matchSelect.setValue(matchList.get(0));
    }

    private void fillSeasonSelectWithData() {
        seasonList.clear();
        seasonList.addAll(seasonService.getAllSeasonsSorted());
        seasonSelect.setItems(seasonList);
    }

    private void fillMatchdaySelectWithData(Season season) {
        matchdayList.clear();
        List<Matchday> matchdaysToAdd = onlyMatchdaysWithActivity ?
                matchdayService.getMatchdaysWithActivitySortedOrElseListWithOnlyFirstMatchday(season) : matchdayService.getMatchdaysSorted(season);
        matchdayList.addAll(matchdaysToAdd);
        matchdaySelect.setItems(matchdayList);
    }

    private void fillMatchSelectWithData(Matchday matchday) {
        matchList.clear();
        matchList.addAll(matchService.getMatches(matchday));
        matchSelect.setItems(matchList);
    }

    @Override
    public void setParameter(BeforeEvent event, @WildcardParameter String param) {
        Map<RegularNavigationLevel, Optional<String>> map = RegularNavigationUtils.getParameterMap(param);
        editFlag = NavigationUtils.editFlag(param);
        navigate(map.get(RegularNavigationLevel.SEASON), map.get(RegularNavigationLevel.MATCHDAY), map.get(RegularNavigationLevel.MATCH));
    }

    private void navigate(Optional<String> seasonParam, Optional<String> matchdayParam, Optional<String> matchParam) {
        if (!seasonEnabled()) return;
        Optional<Season> season = NavigationUtils.getObjectFromParam(seasonList, seasonParam);
        if (season.isPresent()) seasonSelect.setValue(season.get());
        else autoselectSeason();

        if (!matchdayEnabled()) return;
        Optional<Matchday> matchday = NavigationUtils.getObjectFromParam(matchdayList, matchdayParam);
        if (matchday.isPresent()) matchdaySelect.setValue(matchday.get());
        else autoselectMatchday();

        if (!matchEnabled()) return;
        Optional<Match> match = NavigationUtils.getObjectFromParam(matchList, matchParam);
        if (match.isPresent()) matchSelect.setValue(match.get());
        else autoselectMatch();
    }

    public List<Season> getSeasonList() {
        return seasonList;
    }

    public List<Matchday> getMatchdayList() {
        return matchdayList;
    }

    public List<Match> getMatchList() {
        return matchList;
    }

    public Select<Season> getSeasonSelect() {
        return seasonSelect;
    }

    public Select<Matchday> getMatchdaySelect() {
        return matchdaySelect;
    }

    public Select<Match> getMatchSelect() {
        return matchSelect;
    }

    public Optional<Matchday> getSelectedMatchday() {
        return matchdaySelect.getOptionalValue();
    }

    public Optional<Season> getSelectedSeason() {
        return seasonSelect.getOptionalValue();
    }

    public Optional<Match> getSelectedMatch() {
        return matchSelect.getOptionalValue();
    }

    public SeasonService getSeasonService() {
        return seasonService;
    }

    public MatchdayService getMatchdayService() {
        return matchdayService;
    }

    public MatchService getMatchService() {
        return matchService;
    }

    @Override
    public String getWildcardParam() {
        return NavigationUtils.getWildcardParam(
                editFlag,
                getSeasonParam().orElse(null),
                getMatchdayParam().orElse(null),
                getMatchParam().orElse(null));
    }

    public Optional<String> getSeasonParam() {
        return getSelectedSeason().map(EntityStringUtils::getSeasonStringForURL);
    }

    public Optional<String> getMatchdayParam() {
        return getSelectedMatchday().map(EntityStringUtils::getMatchdayStringForURL);
    }

    public Optional<String> getMatchParam() {
        return getSelectedMatch().map(EntityStringUtils::getMatchStringForURL);
    }
}
